import os
import conda

from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
from matplotlib.pyplot import MultipleLocator
import numpy as np
import pandas as pd
import networkx as nx
from geopy.distance import geodesic
from geopy.distance import great_circle as GC
from matplotlib import cm
import matplotlib.colors as colors
from matplotlib.colors import ListedColormap, LinearSegmentedColormap
import cartopy.crs as ccrs
from netCDF4 import Dataset
import numpy as np

address = '/home/meng/bak_bupt/newproj/data/lat_lon_36_2178'
lattic = np.loadtxt(address)


#draw dots distribution
fig = plt.figure(figsize=(16, 9))
ax = fig.add_subplot(1, 1, 1, projection=ccrs.Robinson(central_longitude=-180))
ax.set_global()
ax.stock_img()
ax.coastlines()
x, y = (lattic[:,1], lattic[:,0])
ax.scatter(x,y,s=30,c='black',marker = '.',alpha = 1,transform=ccrs.PlateCarree())

plt.show()
plt.close()

#-----------------------------------------------------------------#
#calculate distance
N = len(lattic)
distance_matrix = np.zeros((N,N))
for i in range(0,N,1):
    for j in range(0,N,1):
        distance_matrix[i][j] = geodesic((lattic[i,0], lattic[i,1]), (lattic[j,0], lattic[j,1])).km


#-----------------------------------------------------------------#


begining_point,ending_point = (0,1)

address = '/home/meng/bak_bupt/newproj/data/year/2019_3_TT_links_30d_tau20_728_36_2178.dat'
data = np.loadtxt(address)

distance_max = 1000
DG = nx.DiGraph()
for i in range(0,N,1):
    DG.add_node(i)

for rr in data:
    i,j,Wpos,Wneg, Tmax, Tmin, Tabs, Cmax, Cmin,_ = rr
    i,j = int(i),int(j)
    
    # if (distance_matrix[i,j]<distance_max):
    #     if (Tabs>0):
    #         DG.add_weighted_edges_from([(j,i,abs(1/max(abs(Wpos),abs(Wneg))))])
    #     else:
    #         DG.add_weighted_edges_from([(i,j,abs(1/max(abs(Wpos),abs(Wneg))))])
    if (distance_matrix[i,j]<distance_max):
            
        if (Tabs>0):
                DG.add_weighted_edges_from([(j,i,abs(1/max(abs(Cmax),abs(Cmin))))])
        else:
                DG.add_weighted_edges_from([(i,j,abs(1/max(abs(Cmax),abs(Cmin))))])

path = nx.dijkstra_path(DG, begining_point,ending_point)
outfile=open('/home/meng/bak_bupt/newproj/data/path/path_2019_3_1000km_36_2178icet','w')
for i in path:
    outfile.write('%d \n'%(i))
    outfile.flush()
outfile.close()

def plot_path():
    path=np.loadtxt('/home/meng/bak_bupt/newproj/data/path/path_2019_3_1000km_36_2178')
    
    address = '/home/meng/bak_bupt/newproj/data/lat_lon_94_2030'
    lattic = np.loadtxt(address)

    m = Basemap(projection='npstere',boundinglat=-10,lon_0=180,resolution='l')
    
    #m.shadedrelief(scale = 0.3)
    m.drawcoastlines(linewidth = 0.5)
    m.drawmeridians(np.arange(-180, 180, 45),dashes=[5, 5],labels=[1,0,0,1],color='#666b6c',linewidth=0.8,size=20,family = 'Arial',zorder=1)
    m.drawparallels(np.arange(-90, 90.001, 20),dashes=[5, 5],labels=[1,0,0,1],color='#666b6c',linewidth=0.8,size=20,family = 'Arial',zorder=1)
    
    x_p = []
    y_p = []
    for i in path:   
        i=int(i)
        x,y = (lattic[i,1],lattic[i,0])
        if x>180:
            x = x-360
        x,y = m(x,y)
        x_p.append(x)
        y_p.append(y)
    m.plot(x_p,y_p,'--',linewidth=2.8,color='#a64036',alpha = 0.6,label='Part1',zorder=4)
    for i in range(0,len(x_p),1):
        m.plot(x_p[i],y_p[i], marker = 'o', color='#a64036',markersize = 7.5,zorder=4)
    m.plot(x_p[0],y_p[0], marker = 'o', color='r',markersize =15,zorder=4)
    m.plot(x_p[-1],y_p[-1], marker = 'o', color='b',markersize = 15,zorder=4)
    #m.scatter(x_p[-1],y_p[-1], marker = 'o', color='#a64036',s = 12,zorder=4)
    
    plt.annotate('', xy=(x_p[5],y_p[5]), xycoords='data',
            xytext=(x_p[4], y_p[4]), textcoords='data',
                        size=6,
                         ha='center', va='bottom',color='#a64036',
                 
            arrowprops=dict(arrowstyle="fancy,head_width=3.5,head_length=3.5",lw=4,fc="#a64036",ec="#a64036",
                                        connectionstyle="arc3,rad=-0.1",alpha=0.9),)
    
   

    ax1 = plt.subplot(111)
    wind_data = Dataset('/home/meng/bak_bupt/newproj/data/2019_3_1_500hPa_850_wind.nc','r',format='NETCDF4')
    wind_data = Dataset('/home/meng/bak_bupt/newproj/data/2019_3_15_500hpa_850hpa_wind.nc','r',format='NETCDF4')
   # wind_data = Dataset('/home/meng/bak_bupt/newproj/data/2010_2_10_500hpa_850hpa_wind.nc','r',format='NETCDF4')


    #V_data = Dataset('/home/jingfang/CLUSTER_DATA/data/NCEP1/Pressure_Levels_Wind/20140812V.nc','r',format='NETCDF4')
    level1=0
    DAY=0
    res=2
    uwnd=wind_data.variables['u'][DAY,level1,][::res,::res]
    vwnd=wind_data.variables['v'][DAY,level1,][::res,::res]
   # hwnd=wind_data.variables['w'][DAY,level1,][::res,::res]
    
    
    lonX  =wind_data.variables['longitude'][:][::res]
    lat  =wind_data.variables['latitude'][:][::res]   
    speed=np.sqrt(vwnd*vwnd+uwnd*uwnd)*3.6    
    # m = Basemap(projection='robin',lon_0 = -4,resolution='c')
    # m.drawmapboundary(linewidth=3,)
    # m.drawparallels(np.arange(-90.,120.,30.),labels=[True,True,True,True],fontsize=20)
    # m.drawmeridians(np.arange(0.,360.,60.),labels=[False,False,False,False])
    # m.drawcoastlines(linewidth=1.5, linestyle='solid', color='w')


    m = Basemap(projection='npstere',boundinglat=10,lon_0=180,resolution='l')
    
    #m.shadedrelief(scale = 0.3)
    m.drawcoastlines(linewidth = 1,)
    m.drawmeridians(np.arange(-180, 180, 45),dashes=[5, 5],labels=[1,0,0,1],color='#666b6c',linewidth=0.8,size=20,family = 'Arial',zorder=1)
    m.drawparallels(np.arange(-90, 90.001, 20),dashes=[5, 5],labels=[1,0,0,1],color='#666b6c',linewidth=0.8,size=20,family = 'Arial',zorder=1)
 
    
    speed, lon = shiftgrid(180, speed, lonX, start=False) 
    uwnd, lon1 = shiftgrid(180, uwnd, lonX, start=False) 
    vwnd, lon2 = shiftgrid(180, vwnd, lonX, start=False) 

    lon2d, lat2d = np.meshgrid(lon, lat)
    
    x, y = m(lon2d, lat2d)
    z=speed
    z1=np.zeros(len(z[:,0]))
    z1=list(z1)
    for i in range(len(z[0,:])):
        z1+=list(z[:,i])
    a=min(z1)
    b=max(z1)
    b=180
    # a=-1
    # b=1
    
    levels1 = np.linspace(a,b,21)

    #cs = m.contourf(x, y, z,extend='both',levels= levels1,cmap=plt.cm.viridis)
    cs = m.contourf(x, y, z,extend='both',levels= levels1,cmap=plt.cm.YlGnBu_r)
   
    
    cbar = plt.colorbar(cs, orientation='horizontal',shrink=0.5,pad=0.05 )
    label_1 =  np.linspace( a, b, 6)
    cbar.set_ticks(label_1)
    cbar.ax.tick_params(labelsize=20)
    cbar.ax.set_xlabel('km/h',fontsize=20)
    level1=0
    DAY=0
    res=2    
    res=40
    uwnd=wind_data.variables['u'][DAY,level1,][::res,::res]
    vwnd=wind_data.variables['v'][DAY,level1,][::res,::res]
   # hwnd=wind_data.variables['w'][DAY,level1,][::res,::res]
    
    lonX  =wind_data.variables['longitude'][:][::res]
    lat  =wind_data.variables['latitude'][:][::res]   
    speed=np.sqrt(vwnd*vwnd+uwnd*uwnd)*3.6        
    speed, lon = shiftgrid(180, speed, lonX, start=False) 
    uwnd, lon1 = shiftgrid(180, uwnd, lonX, start=False) 
    vwnd, lon2 = shiftgrid(180, vwnd, lonX, start=False) 

    lon2d, lat2d = np.meshgrid(lon, lat)
    urot,vrot,x,y = m.rotate_vector(uwnd,vwnd,lon2d,lat2d,returnxy=True)
    Q=m.quiver(x,y,urot,vrot,color='k',pivot='mid', units='inches', width=0.04, headwidth=15., headlength=12.) #or specify, e.g., width=0.003, scale=400)
    figure = plt.gcf() # get current figure
        #cax = figure.add_axes([0.9, 0.1, 0.03, 0.8])
        #figure.colorbar( cax=cax)
        
        
    path=np.loadtxt('/home/meng/bak_bupt/newproj/data/path/path_2019_3_1000km_36_2178')
    
    address = '/home/meng/bak_bupt/newproj/data/lat_lon_36_2178'
    lattic = np.loadtxt(address)
    x_p = []
    y_p = []
    for i in path:   
        i=int(i)
        x,y = (lattic[i,1],lattic[i,0])
        if x>180:
            x = x-360
        x,y = m(x,y)
        x_p.append(x)
        y_p.append(y)
    m.plot(x_p,y_p,'--',linewidth=3.5,color='red',alpha = 0.9,label='Part1',zorder=4)
    for i in range(0,len(x_p),1):
        m.plot(x_p[i],y_p[i], marker = 'o', color='red',markersize = 8.5,zorder=4)
    m.plot(x_p[0],y_p[0], marker = 'o', color='cyan',markersize =25,zorder=4)
    m.plot(x_p[-1],y_p[-1], marker = 'o', color='cyan',markersize = 25,zorder=4)
    #m.scatter(x_p[-1],y_p[-1], marker = 'o', color='#a64036',s = 12,zorder=4)
    
    plt.annotate('', xy=(x_p[5],y_p[5]), xycoords='data',
            xytext=(x_p[4], y_p[4]), textcoords='data',
                        size=6,
                         ha='center', va='bottom',color='#a64036',
                 
            arrowprops=dict(arrowstyle="fancy,head_width=4,head_length=4",lw=4,fc="red",ec="red",
                                        connectionstyle="arc3,rad=-0.1",alpha=1.0),)
    
    
    path=np.loadtxt('/home/meng/bak_bupt/newproj/data/path/path_2019_3_1000km_36_2178icet')
    
    address = '/home/meng/bak_bupt/newproj/data/lat_lon_36_2178'
    lattic = np.loadtxt(address)
    x_p = []
    y_p = []
    for i in path:   
        i=int(i)
        x,y = (lattic[i,1],lattic[i,0])
        if x>180:
            x = x-360
        x,y = m(x,y)
        x_p.append(x)
        y_p.append(y)
    m.plot(x_p,y_p,'--',linewidth=3.5,color='purple',alpha = 0.9,label='Part1',zorder=4)
    for i in range(0,len(x_p),1):
        m.plot(x_p[i],y_p[i], marker = 'o', color='purple',markersize = 8.5,zorder=4)
    m.plot(x_p[0],y_p[0], marker = 'o', color='cyan',markersize =25,zorder=4)
    m.plot(x_p[-1],y_p[-1], marker = 'o', color='cyan',markersize = 25,zorder=4)
    #m.scatter(x_p[-1],y_p[-1], marker = 'o', color='#a64036',s = 12,zorder=4)
    
    plt.annotate('', xy=(x_p[5],y_p[5]), xycoords='data',
            xytext=(x_p[4], y_p[4]), textcoords='data',
                        size=6,
                         ha='center', va='bottom',color='#a64036',
                 
            arrowprops=dict(arrowstyle="fancy,head_width=4,head_length=4",lw=4,fc="purple",ec="purple",
                                        connectionstyle="arc3,rad=-0.1",alpha=1.0),)
    
    
    # plt.annotate('', xy=(x_p[35],y_p[35]), xycoords='data',
    #         xytext=(x_p[34], y_p[34]), textcoords='data',
    #                     size=6,
    #                      ha='center', va='bottom',color='#a64036',
                 
    #         arrowprops=dict(arrowstyle="fancy,head_width=3.5,head_length=3.5",lw=4,fc="red",ec="red",
    #                                     connectionstyle="arc3,rad=-0.1",alpha=1.0),)
    

    figure.set_size_inches(22, 18)
        
    #plt.savefig('/home/meng/bak_bupt/newproj/fig/20190315_path_500_728.pdf',format='pdf',bbox_inches='tight',dpi=600) 
  




#-----------------------------------------------------------------#
plt.figure(figsize=(16,9))
m = Basemap(projection='robin', lon_0=0)
m.shadedrelief(scale = 0.5)
m.drawcoastlines(linewidth = 0.5)
m.drawmeridians(np.arange(-180, 180, 60),dashes=[5, 5],labels=[1,0,0,1],color='#666b6c',linewidth=0.8,alpha=0.8,size=20,family = 'Arial',zorder=1)
m.drawparallels(np.arange(-90, 90.001, 30),dashes=[5, 5],labels=[1,0,0,1],color='#666b6c',linewidth=0.8,alpha=0.8,size=20,family = 'Arial',zorder=1)

x_p = []
y_p = []
for i in path:
    x,y = m(lattic[i,1],lattic[i,0])
    x_p.append(x)
    y_p.append(y)
m.plot(x_p,y_p,'--',linewidth=2.8,color='red',alpha = 0.9,zorder=4)
for i in range(0,len(x_p),1):
    m.plot(x_p[i],y_p[i], marker = 'o', color='red',markersize = 7.5,zorder=4)

m.plot(x_p[-1],y_p[-1], marker = 'o', color='red',markersize = 12,zorder=4)
    
start = m(lattic[begining_point,1],lattic[begining_point,0])
end = m(lattic[ending_point,1],lattic[ending_point,0])


dx = end[0]-start[0]
dy = end[1]-start[1]
m.quiver(start[0],start[1],dx,dy,ls='--',width=0.003,scale=1,scale_units ='xy',alpha=0.5,headwidth=10,headlength=15,headaxislength=10,color='black',zorder=4)

plt.title(r'$year='+str(year)+'$',fontsize=15,family = 'Arial')

plt.show()
plt.close()

#-----------------------------------------------------------------------#
            
plt.figure(figsize=(16,9))
m = Basemap(projection='npstere',boundinglat=0,lon_0=180,resolution='l')

m.shadedrelief(scale = 0.3)
m.drawcoastlines(linewidth = 0.5)
m.drawmeridians(np.arange(-180, 180, 45),dashes=[5, 5],labels=[1,0,0,1],color='#666b6c',linewidth=0.8,size=20,family = 'Arial',zorder=1)
m.drawparallels(np.arange(-90, 90.001, 20),dashes=[5, 5],labels=[1,0,0,1],color='#666b6c',linewidth=0.8,size=20,family = 'Arial',zorder=1)

x_p = []
y_p = []
for i in path:   
    x,y = (lattic[i,1],lattic[i,0])
    if x>180:
        x = x-360
    x,y = m(x,y)
    x_p.append(x)
    y_p.append(y)
m.plot(x_p,y_p,'--',linewidth=2.8,color='#a64036',alpha = 0.6,label='Part1',zorder=4)
for i in range(0,len(x_p),1):
    m.plot(x_p[i],y_p[i], marker = 'o', color='#a64036',markersize = 7.5,zorder=4)
m.plot(x_p[0],y_p[0], marker = 'o', color='#a64036',markersize =12,zorder=4)
m.plot(x_p[-1],y_p[-1], marker = 'o', color='#a64036',markersize = 12,zorder=4)


plt.show()
plt.close()
#-----------------------------------------------------------------#
plt.figure(figsize=(16,9))
h=15000
m = Basemap(projection='nsper',lon_0=160,lat_0=0,satellite_height=h*1000.,resolution='l')

m.shadedrelief(scale = 0.3)
m.drawcoastlines(linewidth = 0.5)
m.drawmeridians(np.arange(-180, 180, 45),dashes=[5, 5],labels=[1,0,0,1],color='#666b6c',linewidth=0.8,size=20,family = 'Arial',zorder=1)
m.drawparallels(np.arange(-90, 90.001, 20),dashes=[5, 5],labels=[1,0,0,1],color='#666b6c',linewidth=0.8,size=20,family = 'Arial',zorder=1)

x_p = []
y_p = []
for i in path:   
    x,y = (lattic[i,1],lattic[i,0])
    if x>180:
        x = x-360
    x,y = m(x,y)
    x_p.append(x)
    y_p.append(y)
m.plot(x_p,y_p,'--',linewidth=2.8,color='#a64036',alpha = 0.6,label='Part1',zorder=4)
for i in range(0,len(x_p),1):
    m.plot(x_p[i],y_p[i], marker = 'o', color='#a64036',markersize = 7.5,zorder=4)
m.plot(x_p[0],y_p[0], marker = 'o', color='#a64036',markersize =12,zorder=4)
m.plot(x_p[-1],y_p[-1], marker = 'o', color='#a64036',markersize = 12,zorder=4)

plt.show()
plt.close()